@@ -55,7 +55,7 @@ class AllocationCandidates(object):
         providers matching a set of supplied resource constraints, with a set
         of allocation requests constructed from that list of resource
         providers. If CONF.placement.randomize_allocation_candidates (on
-        contex.config) is True (default is False) then the order of the
+        context.config) is True (default is False) then the order of the
         allocation requests will be randomized.
 
         :param context: placement.context.RequestContext object.
@@ -526,7 +526,7 @@ def _build_provider_summaries(context, rw_ctx, root_ids, prov_traits):
     # ProviderSummary objects containing one or more ProviderSummaryResource
     # objects representing the resources the provider has inventory for.
     for usage in usages:
-        rp_id = usage['resource_provider_id']
+        rp_id = usage.resource_provider_id
         summary = rw_ctx.summaries_by_id.get(rp_id)
         if not summary:
             pids = provider_ids[rp_id]
@@ -549,7 +549,7 @@ def _build_provider_summaries(context, rw_ctx, root_ids, prov_traits):
             summary.traits = prov_traits[rp_id]
             rw_ctx.summaries_by_id[rp_id] = summary
 
-        rc_id = usage['resource_class_id']
+        rc_id = usage.resource_class_id
         if rc_id is None:
             # NOTE(tetsuro): This provider doesn't have any inventory itself.
             # But we include this provider in summaries since another
@@ -557,20 +557,20 @@ def _build_provider_summaries(context, rw_ctx, root_ids, prov_traits):
             # Let's skip the following and leave "ProviderSummary.resources"
             # field empty.
             continue
-        # NOTE(jaypipes): usage['used'] may be None due to the LEFT JOIN of
+        # NOTE(jaypipes): usage.used may be None due to the LEFT JOIN of
         # the usages subquery, so we coerce NULL values to 0 here. It may
         # also be a Decimal, as that's the type that mysql tends to return
         # when func.sum is used in a query. We need an int, otherwise later
         # JSON serialization will not work.
-        used = int(usage['used'] or 0)
-        allocation_ratio = usage['allocation_ratio']
-        cap = int((usage['total'] - usage['reserved']) * allocation_ratio)
+        used = int(usage.used or 0)
+        allocation_ratio = usage.allocation_ratio
+        cap = int((usage.total - usage.reserved) * allocation_ratio)
         rc_name = context.rc_cache.string_from_id(rc_id)
         rpsr = ProviderSummaryResource(
             resource_class=rc_name,
             capacity=cap,
             used=used,
-            max_unit=usage['max_unit'],
+            max_unit=usage.max_unit,
         )
         # Construct a dict, keyed by resource provider + resource class, of
         # ProviderSummaryResource. This will be used to do a final capacity
@@ -933,16 +933,16 @@ def _provider_ids_from_root_ids(context, root_ids):
     # FROM resource_providers AS rp
     # WHERE rp.root_provider_id IN ($root_ids)
     me = sa.alias(_RP_TBL, name="me")
-    cols = [
+    sel = sa.select(
         me.c.id,
         me.c.uuid,
         me.c.parent_provider_id.label('parent_id'),
         me.c.root_provider_id.label('root_id'),
-    ]
-    sel = sa.select(cols).where(
-        me.c.root_provider_id.in_(sa.bindparam('root_ids', expanding=True)))
+    ).where(
+        me.c.root_provider_id.in_(sa.bindparam('root_ids', expanding=True))
+    )
 
     ret = {}
     for r in context.session.execute(sel, {'root_ids': list(root_ids)}):
-        ret[r['id']] = r
+        ret[r.id] = r
     return ret
