@@ -11,10 +11,10 @@
 #    under the License.
 
 
-import mock
+from unittest import mock
+
 from oslo_utils.fixture import uuidsentinel as uuids
 from oslo_utils import timeutils
-import six
 
 from placement.objects import allocation as alloc_obj
 from placement.objects import resource_provider as rp_obj
@@ -23,7 +23,7 @@ from placement.tests.unit.objects import base
 
 _RESOURCE_PROVIDER_ID = 1
 _RESOURCE_PROVIDER_UUID = uuids.resource_provider
-_RESOURCE_PROVIDER_NAME = six.text_type(uuids.resource_name)
+_RESOURCE_PROVIDER_NAME = str(uuids.resource_name)
 _RESOURCE_CLASS_ID = 2
 _ALLOCATION_ID = 2
 _ALLOCATION_DB = {
@@ -48,6 +48,7 @@ _ALLOCATION_BY_CONSUMER_DB = {
     'resource_class_id': _RESOURCE_CLASS_ID,
     'consumer_uuid': uuids.fake_instance,
     'consumer_id': 1,
+    'consumer_type_id': 1,
     'consumer_generation': 0,
     'used': 8,
     'user_id': 1,
